from pyconnector.driver_utils import get_driver
import pyodbc

class DatabricksODBCConnector:
    def __init__(self, token, workspace_url, http_path, version=None):
        driver_path = get_driver("databricks", "odbc", version)
        conn_str = (
            f"Driver={{{driver_path}}};Host={workspace_url};HTTPPath={http_path};"
            f"AuthMech=3;UID=token;PWD={token};"
        )
        self.conn = pyodbc.connect(conn_str, autocommit=True)

    def cursor(self):
        return self.conn.cursor()

    def close(self):
        self.conn.close()


import smtplib
from email.message import EmailMessage

class SMTPConnector:
    def __init__(self, host, port, username, password, use_tls=True):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.use_tls = use_tls

    def send_email(self, to_address, subject, content):
        msg = EmailMessage()
        msg["From"] = self.username
        msg["To"] = to_address
        msg["Subject"] = subject
        msg.set_content(content)

        with smtplib.SMTP(self.host, self.port) as server:
            if self.use_tls:
                server.starttls()
            server.login(self.username, self.password)
            server.send_message(msg)


from pyconnector.driver_utils import get_driver
import jaydebeapi

class PostgresJDBCConnector:
    def __init__(self, host, port, database, user, password, version=None):
        jar_path = get_driver("postgres", "jdbc", version)
        self.conn = jaydebeapi.connect(
            "org.postgresql.Driver",
            f"jdbc:postgresql://{host}:{port}/{database}",
            [user, password],
            jar_path
        )

    def cursor(self):
        return self.conn.cursor()

    def close(self):
        self.conn.close()


import requests

class PostgresAPIConnector:
    def __init__(self, base_url, token):
        self.session = requests.Session()
        self.session.headers.update({"Authorization": f"Bearer {token}"})
        self.base_url = base_url.rstrip("/")

    def get(self, endpoint, params=None):
        res = self.session.get(f"{self.base_url}{endpoint}", params=params)
        res.raise_for_status()
        return res.json()

    def post(self, endpoint, data=None):
        res = self.session.post(f"{self.base_url}{endpoint}", json=data)
        res.raise_for_status()
        return res.json()



import paramiko

class SFTPConnector:
    def __init__(self, host, port, username, password):
        self.transport = paramiko.Transport((host, port))
        self.transport.connect(username=username, password=password)
        self.sftp = paramiko.SFTPClient.from_transport(self.transport)

    def upload_file(self, local_path, remote_path):
        self.sftp.put(local_path, remote_path)

    def download_file(self, remote_path, local_path):
        self.sftp.get(remote_path, local_path)

    def close(self):
        self.sftp.close()
        self.transport.close()


from pyconnector.driver_utils import get_driver
import pyodbc

class PostgresODBCConnector:
    def __init__(self, host, port, database, user, password, version=None):
        driver_path = get_driver("postgres", "odbc", version)
        conn_str = (
            f"Driver={{{driver_path}}};Server={host};Port={port};Database={database};"
            f"UID={user};PWD={password};"
        )
        self.conn = pyodbc.connect(conn_str, autocommit=True)

    def cursor(self):
        return self.conn.cursor()

    def close(self):
        self.conn.close()



from .api_connector import DatabricksAPIConnector
from .jdbc_connector import DatabricksJDBCConnector
from .odbc_connector import DatabricksODBCConnector

__all__ = [
    "DatabricksAPIConnector",
    "DatabricksJDBCConnector",
    "DatabricksODBCConnector"
]




from .api_connector import PostgresAPIConnector
from .jdbc_connector import PostgresJDBCConnector
from .odbc_connector import PostgresODBCConnector

__all__ = [
    "PostgresAPIConnector",
    "PostgresJDBCConnector",
    "PostgresODBCConnector"
]



try:
    import jaydebeapi
except ImportError:
    jaydebeapi = None

class DatabricksJDBCConnector:
    def __init__(self, token, workspace_url, http_path, version=None):
        jar_path = get_driver("databricks", "jdbc", version)
        self.conn = jaydebeapi.connect(
            "com.databricks.client.jdbc.Driver",
            f"jdbc:databricks://{workspace_url}/;transportMode=http;ssl=1;AuthMech=3;httpPath={http_path};UID=token;PWD={token}",
            ["token", token],
            jar_path
        )

    def cursor(self):
        return self.conn.cursor()

    def close(self):
        self.conn.close()


import requests

class DatabricksAPIConnector:
    def __init__(self, token: str, workspace_url: str):
        if not token or not workspace_url:
            raise ValueError("token and workspace_url required.")
        self.session = requests.Session()
        self.session.headers.update({
            "Authorization": f"Bearer {token.strip()}"
        })
        self.workspace_url = workspace_url.rstrip("/")

    def get(self, endpoint: str, params=None):
        res = self.session.get(f"{self.workspace_url}{endpoint}", params=params)
        res.raise_for_status()
        return res.json()

    def post(self, endpoint: str, data=None):
        res = self.session.post(f"{self.workspace_url}{endpoint}", json=data)
        res.raise_for_status()
        return res.json()



import os
import glob

DRIVER_BASE = os.path.abspath(os.path.join(os.path.dirname(__file__), "drivers"))

def get_driver(system, driver_type="jdbc", version=None):
    driver_dir = os.path.join(DRIVER_BASE, driver_type)
    if driver_type == "jdbc":
        pattern = f"{system}-{driver_type}-{version}*.jar" if version else f"{system}-{driver_type}-*.jar"
    elif driver_type == "odbc":
        pattern = f"{system}-{driver_type}-{version}*.so" if version else f"{system}-{driver_type}-*.so"
    else:
        raise ValueError("driver_type must be 'jdbc' or 'odbc'")

    candidates = sorted(glob.glob(os.path.join(driver_dir, pattern)), reverse=True)
    if not candidates:
        raise FileNotFoundError(f"No driver found for {system} ({driver_type}, version={version})")
    return candidates[0]